import {
	JupyterFrontEnd, JupyterFrontEndPlugin
} from '@jupyterlab/application';

import {
	NotebookPanel, INotebookTracker, Notebook
} from '@jupyterlab/notebook';

import {
	Cell, CodeCell, ICellModel
} from '@jupyterlab/cells';


import './codemirror-finesse-python';

const plugin: JupyterFrontEndPlugin<void> = {
	activate,
	id: 'finesse-syntax-jupyter',
	requires: [INotebookTracker],
	autoStart: true
};

function activate(app: JupyterFrontEnd, tracker: INotebookTracker) {
	let style = document.createElement('style');
	style.type = 'text/css';
	style.innerHTML = '.finesse { background-color: rgba(0, 0, 0, 0.04); }';
	document.getElementsByTagName('head')[0].appendChild(style);
	tracker.widgetAdded.connect(activate_finesse)
};

function check_all(sender: NotebookPanel): void {
	sender.content.widgets.forEach(cell => {
		if (cell instanceof CodeCell && cell.model.mimeType != "text/x-finesse-python") {
			cell.model.mimeType = "text/x-finesse-python";
		}
	});
};

function set_mime(model: ICellModel) {
	// Jupyter seems to have some weirdness related to setup of the
	// mime type of each cell. To avoid having timeouts everywhere,
	// everytime we select a new cell, we attach a function that checks the
	// mime type on each content change and sets it if required, otherwise
	// disconnects itself.
	if (model.mimeType == "text/x-finesse-python") {
		model.contentChanged.disconnect(set_mime);
	} else {
		model.mimeType = "text/x-finesse-python";
	}
}

function check_cell(sender: Notebook, cell: Cell): void {
	if (cell instanceof CodeCell) {
		cell.model.contentChanged.connect(set_mime);
	}
}

function activate_finesse(sender: INotebookTracker, panel: NotebookPanel): void {
	panel.content.activeCellChanged.connect(check_cell);

	// I ain't proud of this, but it's the sanest way I could find to get
	// the syntax highlighting to change on startup without selecting a
	// different cell
	setTimeout(() => {check_all(panel)}, 500);
	setTimeout(() => {check_all(panel)}, 2000);
	setTimeout(() => {check_all(panel)}, 5000);
};

export default plugin;
